"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const server_1 = require("react-dom/server");
const html_react_parser_1 = __importDefault(require("html-react-parser"));
const domelementtype_1 = require("domelementtype");
const transformers_1 = require("./transformers");
const utils_1 = require("./utils");
const transformers = [
    transformers_1.UnitValueTransformer,
    transformers_1.DivTransformer,
    transformers_1.ViewTransformer,
    transformers_1.SvgTransformer,
    transformers_1.LineTransformer,
    transformers_1.PolylineTransformer,
    transformers_1.PolygonTransformer,
    transformers_1.PathTransformer,
    transformers_1.RectTransformer,
    transformers_1.CircleTransformer,
    transformers_1.EllipseTransformer,
    transformers_1.TextTransformer,
    transformers_1.TspanTransformer,
    transformers_1.SpanTransformer,
    transformers_1.GTransformer,
    transformers_1.StopTransformer,
    transformers_1.DefsTransformer,
    transformers_1.ClipPathTransformer,
    transformers_1.LinearGradientTransformer,
    transformers_1.RadialGradientTransformer,
    transformers_1.UlTransformer,
    transformers_1.LiTransformer,
];
function SvgWrapper(props) {
    const { reachartResponsive = true, baseFontSize = 11, propsUpdateQuery: elementProps = {}, styleUpdateQuery: elementStyles = {} } = props;
    let html = (0, server_1.renderToStaticMarkup)(props.children);
    const parserOptions = {
        transform(reactNode, domNode, index) {
            var _a, _b;
            switch (domNode.type) {
                case domelementtype_1.ElementType.Text:
                    return domNode.data;
                case domelementtype_1.ElementType.Tag:
                    const element = reactNode;
                    const data = {
                        element,
                        children: element.props.children,
                        domNode,
                        index,
                        key: element.key || index.toString(),
                        settings: {
                            baseFontSize
                        }
                    };
                    for (const transformer of transformers) {
                        data.element = transformer(data);
                        data.children = (_b = (_a = data.element) === null || _a === void 0 ? void 0 : _a.props) === null || _b === void 0 ? void 0 : _b.children;
                    }
                    const transformed = data.element != null && data.element.type != reactNode.type;
                    return transformed ? data.element : null;
                default:
                    return null;
            }
        }
    };
    let svgElement = (0, html_react_parser_1.default)(html, parserOptions);
    const defaultElementStyles = {
        'tspan, text, span': {
            fontSize: baseFontSize
        },
        '.recharts-legend-wrapper': {
            position: undefined,
            width: undefined,
            flexDirection: 'row',
            justifyContent: 'center',
            alignItems: 'center'
        },
        '.recharts-default-legend': {
            flexDirection: 'row',
            justifyContent: 'center',
            flexWrap: 'wrap',
            gap: 8,
        },
        '.recharts-legend-item': {
            flexDirection: 'row',
            gap: 4
        },
    };
    if (reachartResponsive) {
        defaultElementStyles['.recharts-wrapper, .recharts-wrapper > svg'] = {
            width: '100%',
            height: 'auto'
        };
    }
    const updatePropsQueries = Object.entries(elementProps).concat(Object.entries(defaultElementStyles)
        .concat(Object.entries(elementStyles))
        .map(([selector, style]) => ([selector, { style }])));
    updatePropsQueries.forEach(([selector, props]) => {
        svgElement = (0, utils_1.updateProperties)(svgElement, selector, props);
    });
    return svgElement;
}
exports.default = SvgWrapper;
