"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LiTransformer = exports.UlTransformer = exports.RadialGradientTransformer = exports.LinearGradientTransformer = exports.ClipPathTransformer = exports.DefsTransformer = exports.StopTransformer = exports.GTransformer = exports.TitleTransformer = exports.SpanTransformer = exports.TspanTransformer = exports.TextTransformer = exports.EllipseTransformer = exports.CircleTransformer = exports.RectTransformer = exports.PathTransformer = exports.PolygonTransformer = exports.PolylineTransformer = exports.LineTransformer = exports.SvgTransformer = exports.ViewTransformer = exports.DivTransformer = exports.UnitValueTransformer = void 0;
const react_1 = __importDefault(require("react"));
const renderer_1 = require("@react-pdf/renderer");
const utils_1 = require("./utils");
const BaseNodeTransformer = (nodeName, transformer) => {
    return (data) => {
        if (data.domNode.name != nodeName)
            return data.element;
        return transformer(data);
    };
};
const UnitValueTransformer = (data) => {
    if (!data.element)
        return data.element;
    const props = Object.assign({}, data.element.props);
    if (props.style) {
        Object.entries(props.style).filter(([prop, value]) => value != null).forEach(([prop, value]) => {
            props.style[prop] = (0, utils_1.convertUnitValue)(value, data.settings.baseFontSize);
        });
    }
    if (props.strokeDasharray) {
        props.strokeDasharray = (0, utils_1.convertUnitValue)(props.strokeDasharray.split(' ')[0], data.settings.baseFontSize).toString();
    }
    if (data.domNode.name == 'tspan') {
        props.dx = (0, utils_1.convertUnitValue)(props.dx || 0, data.settings.baseFontSize);
        props.dy = (0, utils_1.convertUnitValue)(props.dy || 0, data.settings.baseFontSize);
    }
    return react_1.default.cloneElement(data.element, props, data.children);
};
exports.UnitValueTransformer = UnitValueTransformer;
exports.DivTransformer = BaseNodeTransformer('div', (data) => { var _a; return react_1.default.createElement(renderer_1.View, Object.assign({}, (_a = data.element) === null || _a === void 0 ? void 0 : _a.props, { key: data.key }), data.children); });
exports.ViewTransformer = BaseNodeTransformer('view', (data) => { var _a; return react_1.default.createElement(renderer_1.View, Object.assign({}, (_a = data.element) === null || _a === void 0 ? void 0 : _a.props, { key: data.key }), data.children); });
exports.SvgTransformer = BaseNodeTransformer('svg', (data) => { var _a; return react_1.default.createElement(renderer_1.Svg, Object.assign({}, (_a = data.element) === null || _a === void 0 ? void 0 : _a.props, { key: data.key }), data.children); });
exports.LineTransformer = BaseNodeTransformer('line', (data) => { var _a; return react_1.default.createElement(renderer_1.Line, Object.assign({}, (_a = data.element) === null || _a === void 0 ? void 0 : _a.props, { key: data.key }), data.children); });
exports.PolylineTransformer = BaseNodeTransformer('polyline', (data) => { var _a; return react_1.default.createElement(renderer_1.Polyline, Object.assign({}, (_a = data.element) === null || _a === void 0 ? void 0 : _a.props, { key: data.key }), data.children); });
exports.PolygonTransformer = BaseNodeTransformer('polygon', (data) => { var _a; return react_1.default.createElement(renderer_1.Polygon, Object.assign({}, (_a = data.element) === null || _a === void 0 ? void 0 : _a.props, { key: data.key }), data.children); });
exports.PathTransformer = BaseNodeTransformer('path', (data) => { var _a; return react_1.default.createElement(renderer_1.Path, Object.assign({}, (_a = data.element) === null || _a === void 0 ? void 0 : _a.props, { key: data.key }), data.children); });
exports.RectTransformer = BaseNodeTransformer('rect', (data) => { var _a; return react_1.default.createElement(renderer_1.Rect, Object.assign({}, (_a = data.element) === null || _a === void 0 ? void 0 : _a.props, { key: data.key }), data.children); });
exports.CircleTransformer = BaseNodeTransformer('circle', (data) => { var _a; return react_1.default.createElement(renderer_1.Circle, Object.assign({}, (_a = data.element) === null || _a === void 0 ? void 0 : _a.props, { key: data.key }), data.children); });
exports.EllipseTransformer = BaseNodeTransformer('ellipse', (data) => { var _a; return react_1.default.createElement(renderer_1.Ellipse, Object.assign({}, (_a = data.element) === null || _a === void 0 ? void 0 : _a.props, { key: data.key }), data.children); });
exports.TextTransformer = BaseNodeTransformer('text', (data) => {
    var _a;
    const children = react_1.default.Children.map(data.children, child => {
        var _a, _b;
        if ((child === null || child === void 0 ? void 0 : child.type) != 'TSPAN')
            return child;
        const coordinates = {};
        if (child.props.dx) {
            coordinates.x = child.props.dx + Number.parseFloat((_a = data.element) === null || _a === void 0 ? void 0 : _a.props.x);
        }
        if (child.props.dy) {
            coordinates.y = child.props.dy + Number.parseFloat((_b = data.element) === null || _b === void 0 ? void 0 : _b.props.y);
        }
        if (coordinates.x || coordinates.y) {
            return react_1.default.cloneElement(child, coordinates);
        }
        return child;
    });
    const { props } = data.element;
    const _b = props, { x, y, style = {} } = _b, otherProps = __rest(_b, ["x", "y", "style"]);
    if ((_a = props.transform) === null || _a === void 0 ? void 0 : _a.startsWith('rotate(')) {
        const mathes = props.transform.matchAll(new RegExp('[-.0-9]+', 'g'));
        const [rotate, posX = x, posY = y] = Array.from(mathes).map((value) => value[0]);
        style.transformOrigin = `${posX} ${posY}`;
    }
    return react_1.default.createElement(renderer_1.Text, Object.assign({ x: x, y: y, style: style }, otherProps, { key: data.key }), children);
});
exports.TspanTransformer = BaseNodeTransformer('tspan', (data) => { var _a; return react_1.default.createElement(renderer_1.Tspan, Object.assign({}, (_a = data.element) === null || _a === void 0 ? void 0 : _a.props, { key: data.key }), data.children); });
exports.SpanTransformer = BaseNodeTransformer('span', (data) => { var _a; return react_1.default.createElement(renderer_1.Text, Object.assign({}, (_a = data.element) === null || _a === void 0 ? void 0 : _a.props, { key: data.key }), data.children); });
exports.TitleTransformer = BaseNodeTransformer('title', (data) => { var _a; return react_1.default.createElement(renderer_1.Text, Object.assign({}, (_a = data.element) === null || _a === void 0 ? void 0 : _a.props, { key: data.key }), data.children); });
exports.GTransformer = BaseNodeTransformer('g', (data) => { var _a; return react_1.default.createElement(renderer_1.G, Object.assign({}, (_a = data.element) === null || _a === void 0 ? void 0 : _a.props, { key: data.key }), data.children); });
exports.StopTransformer = BaseNodeTransformer('stop', (data) => { var _a; return react_1.default.createElement(renderer_1.Stop, Object.assign({}, (_a = data.element) === null || _a === void 0 ? void 0 : _a.props, { key: data.key }), data.children); });
exports.DefsTransformer = BaseNodeTransformer('defs', (data) => { var _a; return react_1.default.createElement(renderer_1.Defs, Object.assign({}, (_a = data.element) === null || _a === void 0 ? void 0 : _a.props, { key: data.key }), data.children); });
exports.ClipPathTransformer = BaseNodeTransformer('clippath', (data) => { var _a; return react_1.default.createElement(renderer_1.ClipPath, Object.assign({}, (_a = data.element) === null || _a === void 0 ? void 0 : _a.props, { key: data.key }), data.children); });
exports.LinearGradientTransformer = BaseNodeTransformer('lineargradient', (data) => { var _a; return react_1.default.createElement(renderer_1.LinearGradient, Object.assign({}, (_a = data.element) === null || _a === void 0 ? void 0 : _a.props, { key: data.key }), data.children); });
exports.RadialGradientTransformer = BaseNodeTransformer('radialgradient', (data) => { var _a; return react_1.default.createElement(renderer_1.RadialGradient, Object.assign({}, (_a = data.element) === null || _a === void 0 ? void 0 : _a.props, { key: data.key }), data.children); });
exports.UlTransformer = BaseNodeTransformer('ul', (data) => { var _a; return react_1.default.createElement(renderer_1.View, Object.assign({}, (_a = data.element) === null || _a === void 0 ? void 0 : _a.props, { key: data.key }), data.children); });
exports.LiTransformer = BaseNodeTransformer('li', (data) => { var _a; return react_1.default.createElement(renderer_1.View, Object.assign({}, (_a = data.element) === null || _a === void 0 ? void 0 : _a.props, { key: data.key }), data.children); });
