"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateProperties = exports.createFlatDom = exports.convertUnitValue = void 0;
const react_1 = __importDefault(require("react"));
const CSSSelectAdapter_1 = __importDefault(require("./CSSSelectAdapter"));
const css_select_1 = __importDefault(require("css-select"));
const deepmerge_1 = __importDefault(require("deepmerge"));
function convertEm(value, baseFontSize) {
    return value * baseFontSize;
}
function convertUnitValue(value, baseFontSize) {
    const stringValue = value.toString();
    if (stringValue.endsWith('%')) {
        return value;
    }
    if (stringValue.endsWith('em')) {
        return convertEm(Number.parseFloat(stringValue.split('em')[0]), baseFontSize);
    }
    if (stringValue.endsWith('px')) {
        return Number.parseFloat(stringValue.split('px')[0]);
    }
    if (Number.isInteger(stringValue)) {
        return Number.parseFloat(stringValue);
    }
    return value;
}
exports.convertUnitValue = convertUnitValue;
function getChildDom(parent) {
    const items = [];
    react_1.default.Children.forEach(parent.element.props.children, (child, index) => {
        if (react_1.default.isValidElement(child)) {
            const item = { element: child, parentElement: parent.element, parent, index };
            items.push(item);
            items.push(...getChildDom(item));
        }
    });
    return items;
}
function createFlatDom(root) {
    const items = [];
    react_1.default.Children.forEach(root, (child, index) => {
        if (react_1.default.isValidElement(child)) {
            const item = { element: child, parentElement: null, parent: null, index };
            items.push(item);
            items.push(...getChildDom(item));
        }
    });
    return items;
}
exports.createFlatDom = createFlatDom;
function updateItemInParent(item) {
    if (item.parent == null)
        return;
    const parent = item.parent.element;
    if (Array.isArray(parent.props.children)) {
        parent.props.children[item.index] = item.element;
    }
    else {
        item.parent.element = react_1.default.cloneElement(parent, parent.props, item.element);
        updateItemInParent(item.parent);
    }
}
function updateProperties(root, selector, props) {
    const dom = createFlatDom(root);
    const items = (0, css_select_1.default)(selector, dom, {
        adapter: new CSSSelectAdapter_1.default(dom)
    });
    const getProps = (currentProps) => {
        if (typeof props === 'function') {
            return props(currentProps);
        }
        return props;
    };
    items.forEach(item => {
        const _a = item.element.props || {}, { children: _ } = _a, baseElementProps = __rest(_a, ["children"]);
        item.element = react_1.default.cloneElement(item.element, deepmerge_1.default.all([baseElementProps, getProps(baseElementProps)]));
        updateItemInParent(item);
    });
    return dom.filter(x => x.parentElement == null).map(x => x.element);
}
exports.updateProperties = updateProperties;
